#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include "ShiftRegister74HC595.h"
#include "DHT11.h"

#define DHT_PIN 6
#define BUZZER_PIN 3
#define SOUND_SENSOR 2
#define RED_LED 4
#define YELLOW_LED 12
#define GREEN_LED 13
#define SOUND_LED A0
#define SETTINGS_BUTTON 9
#define BACK_BUTTON 10
#define NEXT_BUTTON 8
// 2 led digits module
#define TWOLED_DATA 5
#define CLK 7
#define TWOLED_LOAD 11

// Turn Led OFF
#define TLOFF(LED) (digitalWrite(LED, LOW))
// Turn Led ON
#define TLON(LED) (digitalWrite(LED, HIGH))
// Turn ALL LEDS OFF
#define TURNALLOFF() digitalWrite(GREEN_LED, LOW); \
digitalWrite(YELLOW_LED, LOW); \
digitalWrite(RED_LED, LOW); \

// States
#define IDLE 0 
#define SETTINGS_STATE 1
#define LANGUAGE_STATE 2

// Language Settings
#define ENGLISH 0
#define ROMANIAN 1

unsigned char STATE = IDLE;
const unsigned char settingsArrEnglish[][17] = {
  "<- Disp Temp ->", "<- Disp Humid ->", "<- Disp Both ->", "<- Language ->","<- Zen ->"
};
const unsigned char settingsArrRomanian[][17] = {
  "<- Afis Temp ->", "<- Afis Umid ->", "<- Afis Ambele ->", "<- Limba ->","<- Zen ->"
};

const unsigned char languages[][17] = {
  "<- RO ->", "<- EN ->"
};
uint8_t language_idx = 0;
const uint8_t LANGUAGE_ARR_SIZE = sizeof(languages) / sizeof(languages[0]);

// English is default
uint8_t LANGUAGE = ENGLISH;
unsigned char (*settingsArr)[17] = settingsArrEnglish;

const unsigned char SETTINGS_ARR_SIZE = sizeof(settingsArrEnglish) / sizeof(settingsArrEnglish[0]);
unsigned char idx = 0;
unsigned char (*currentOption)[17] = &settingsArr[0];
// Temp by default
unsigned char selectedSetting{0};

// Facem citirea de temp si humid o data la 10 secunde
unsigned long prevMillis = 0; // Store the last time the sensor was read
const uint16_t interval = 10000;

const uint16_t melody[] = {
  392, 440
};

const uint16_t zen_melody[] = {
  196, 220, 247, 262
};

const uint16_t rising_melody[] = {
  262, 294, 330, 349, 392, 440
};

const uint16_t noteDurations[] = {
  500, 500, 500, 500, 500, 500
};
// Set the LCD address to 0x27 for a 16 chars and 2 line display
LiquidCrystal_I2C lcd(0x27, 16, 2);
bool is_backlight_on = true;

DHT11 dht(DHT_PIN);
int t = 0, h = 0;
ShiftRegister74HC595 sr(1, TWOLED_DATA, CLK, TWOLED_LOAD);

// mapare pentru cifrele de pe 2 digits module
uint8_t digits[]{
  0b11000000, // 0
  0b11111001, // 1
  0b10110000, // 2
  0b10110000, // 3
  0b10011001, // 4
  0b10010010,
  0b10000010,
  0b11111000,
  0b10000000,
  0b10010001
};

void temperature_led_handle(uint16_t temp){
  TURNALLOFF();
  if(temp < 25){
    TLON(GREEN_LED);
  }else if(temp >= 25){
    TLON(YELLOW_LED);
  }else if(temp >= 30){
    TLON(RED_LED);
  }
}

void humidty_led_handle(uint16_t hum){
  TURNALLOFF();
  if(hum < 35){
    TLON(GREEN_LED);
  }else if(hum >= 25 && hum < 50){
    TLON(YELLOW_LED);
  }else{
    TLON(RED_LED);
  }

}

void displaySetting(const char* label, int value1, const char* label2 = nullptr, int value2 = 0) {
  lcd.setCursor(0, 1);
  char text[17]{};

  if (label2) {
      sprintf(text, "%s %d, %s %d", label, value1, label2, value2);
  } else {
      sprintf(text, "%s %d", label, value1);
  }

  lcd.print(text);
}

void play_melody(const uint16_t* notes = melody, const uint16_t* durations = noteDurations){
  for(int i = 0; i < 2; ++i){
    tone(BUZZER_PIN, notes[i], durations[i]);
    delay(durations[i] * .1f);
    noTone(BUZZER_PIN);
  }
}

void clear_lcd(uint8_t* to_print){
  lcd.setCursor(0, 0);
  lcd.clear();
  lcd.print((char*)to_print);
}

void setup() {
  // Initialize the LCD
  lcd.init();
  Serial.begin(9600);
  // Turn on the backlight
  lcd.backlight();
  
  // Print "Hello" to the LCD
  lcd.setCursor(0, 0); // Set the cursor to column 0, line 0
  lcd.print("Monitoring...");

  dht.readTemperatureHumidity(t, h);

  // LEDS
  pinMode(RED_LED, OUTPUT);
  digitalWrite(RED_LED, LOW);

  pinMode(YELLOW_LED, OUTPUT);
  digitalWrite(YELLOW_LED, LOW);

  pinMode(GREEN_LED, OUTPUT);
  digitalWrite(GREEN_LED, LOW);

  pinMode(SOUND_LED, OUTPUT);
  digitalWrite(SOUND_LED, HIGH);

  // Toner  
  pinMode(BUZZER_PIN, OUTPUT);

  // NOISE Sensor
  pinMode(SOUND_SENSOR, INPUT);

  // Buttons
  pinMode(SETTINGS_BUTTON, INPUT_PULLUP);
  pinMode(NEXT_BUTTON, INPUT_PULLUP);
  pinMode(BACK_BUTTON, INPUT_PULLUP);

  // two digit module
  uint8_t ar[]{digits[1], digits[2]};
  sr.setAll(ar);
}

unsigned long lastEvent = 0;

void loop() {
  unsigned char buttonState = digitalRead(SETTINGS_BUTTON);
  
  int sensorData = digitalRead(SOUND_SENSOR);

  Serial.println(STATE);

	// If pin goes LOW, sound is detected
	if (sensorData == LOW) {
		
		// If 25ms have passed since last LOW state, it means that
		// the clap is detected and not due to any spurious sounds
		if (millis() - lastEvent > 200) {
			Serial.println("Clap detected!");
      int led_state = digitalRead(SOUND_LED);
      if(led_state == LOW)
        digitalWrite(SOUND_LED, HIGH);
      else
        digitalWrite(SOUND_LED, LOW);
		}
		
		// Remember when last event happened
		lastEvent = millis();
	}

  switch(STATE){
    case IDLE: {
      unsigned long currMillis = millis();

      // Verificam daca au trecut 10 secunde
      if (currMillis - prevMillis >= interval) {
        prevMillis = currMillis;
        dht.readTemperatureHumidity(t, h);
      }

      // Pressed
      if(!buttonState){
        STATE = SETTINGS_STATE;
        lcd.clear();
        delay(250);
      }

      switch(selectedSetting){
        // Temp
        case 0: {
          temperature_led_handle(t);
          displaySetting("Temp", t);
          break;
        }
        // Humidity
        case 1: {
          humidty_led_handle(h);
          displaySetting("Humidity", h);
          break;
        // Both
        }case 2: {
          TURNALLOFF();
          displaySetting("Tmp", t, "Hum", h);
          break;
        }
        default: break;
      }

      break;
    }
    case SETTINGS_STATE: {
      lcd.setCursor(0, 0);
      lcd.print((LANGUAGE == ENGLISH) ? "Settings Menu" : "Meniu Setari");
      lcd.setCursor(0, 1);

      unsigned char nextButtonState = digitalRead(NEXT_BUTTON);
      unsigned char backButtonState = digitalRead(BACK_BUTTON);
      
      if(!nextButtonState){
        idx = (idx + 1) % SETTINGS_ARR_SIZE;
        currentOption = &settingsArr[idx];
        delay(250);
        lcd.clear();
      }else if(!backButtonState){
        idx = (idx <= 0) ? SETTINGS_ARR_SIZE - 1 : (idx - 1) % 3;
        currentOption = &settingsArr[idx];s
        delay(250);
        lcd.clear();
      }

      if(buttonState == 0){
        selectedSetting = idx;
        switch(selectedSetting){
          case 0: {
            play_melody();
            break;
          }
          case 1:
          case 2: {
            play_melody(rising_melody);
            break;
          }
          case SETTINGS_ARR_SIZE - 1: {
            play_melody(zen_melody);
            break;
          }
          default:{
            play_melody();
            break;
          }
        }

        if(selectedSetting == SETTINGS_ARR_SIZE - 2){
          STATE = LANGUAGE_STATE;
          lcd.clear();
          lcd.setCursor(0, 0);
          delay(30);
          break;
        }

        lcd.setCursor(0, 0);
        lcd.clear();
        lcd.print("Monitoring...");
        STATE = IDLE;

        if(idx == SETTINGS_ARR_SIZE - 1){
          Serial.println("plm");
          is_backlight_on = is_backlight_on ? (lcd.noBacklight(), false) : (lcd.backlight(), true);
        }

        // Putin delay, deoarece va iesi din acest state si va intra inapoi instant tot in SETTINGS_STATE
        delay(250);
      }

      lcd.print((char*)*currentOption);
      
      break;
    }
    case LANGUAGE_STATE: {
      lcd.setCursor(0, 0);
      lcd.print(LANGUAGE == 0 ? "Choose Lang" : "Alege limba");

      unsigned char nextButtonState = digitalRead(NEXT_BUTTON);
      unsigned char backButtonState = digitalRead(BACK_BUTTON);
      lcd.setCursor(0, 1);
      lcd.print((char*)languages[language_idx]);

      if(!nextButtonState){
        language_idx = (language_idx + 1) % LANGUAGE_ARR_SIZE;
      }

      if(!buttonState){
        LANGUAGE = (LANGUAGE == ENGLISH) ? ROMANIAN : ENGLISH;
        settingsArr = (settingsArr == settingsArrEnglish) ? settingsArrRomanian : settingsArrEnglish;
        STATE = IDLE;

        clear_lcd("Monitorizare...");
      }

      delay(150);

      break;
    }
    default: {
      break;
    }
  }
}